Unit InsGame;
{ PRAM MODEL WITH FULL CONTROL }

Interface

Uses Dos;

Const
  WordPRAM = 32;
  WordHost = 16;
  MaxRecords =  1; { array  Control's length }
  NumProcessors = WordPRAM * MaxRecords; { 32 }
  CtrlName = -1;

  WordsOnWordsPRAM = WordPRAM div WordHost; { words PC = 2 }

  MemorySize = MaxInt div WordsOnWordsPRAM; { 16383 * 4(*LongInt*) = 64K }
  MaxProcessor = NumProcessors - 1; { 63 }
  MinAddress = 0;
  MaxAddress = MemorySize - 1;

  MaxProg = MaxInt div 2;
  ProgramMemorySize = MaxProg div WordsOnWordsPRAM; { 16383 * 4(*LongInt*) = 64K }
  MinProgramMemoryAddress = 0;
  MaxProgramMemoryAddress = ProgramMemorySize -1;

Type

  OperationPart = (  active2, 
                     add2, 
                     and2, 
                     andInt2, 
                     assign2,
                     broadcast2,                   (*5-5-94*)
                     complement2, 
                     constant2, 
                     copynotops2, 
                     copytops2,
                     debug2, 
                     defaddr2, 
                     defarg2, 
                     divide2, 
                     do2,
                     endblock2, 
                     endNestedPattern2, 
                     endPattern2,
                     endproc2, 
                     endprog2, 
                     equal2, 
                     equalInt2, 
                     endRelax2,
                     endSinglePattern2,
		     field2, 
                     fill2, 
                     fillzero2, 
                     free2,
                     getTime2,
		     globalcall2, 
                     globalcalltwo2, 
                     globalcallthree2,
                     globalvalue2, 
                     globalvaluetwo2, 
                     globalvaluethree2,
                     globalvar2, 
                     globalvartwo2, 
                     globalvarthree2,
		     goifnozero2, 
                     goifzero2, 
                     goto2, 
                     greater2, 
                     greaterInt2,
		     in2, 
                     index2, 
                     initFor2, 
                     initNestedFor2,
		     less2, 
                     lessInt2, 
                     localcall2, 
                     localvalue2, 
                     localvar2,
                     loop2,
		     marked2, 
                     max2, 
                     min2, 
                     Minus2, 
                     modulo2, 
                     multiply2,
                     nestedLoop2,
		     nestedpattern2, 
                     newblock2, 
                     newline2, 
                     not2, 
                     notInt2,
		     notdo2, 
                     notequal2, 
                     notequalInt2,
                     notgreater2, 
                     notgreaterInt2, 
                     notless2, 
                     notlessInt2,
		     or2, 
                     orInt2,
		     pattern2, 
                     poppattern2,
                     privateAssign2, 
                     privateGetTime2,

                     privateGlobalCall12,
                     privateGlobalCall22,
                     privateGlobalCall32,
                     privateLocalCall2,
                     privateNewblock2,
                     privateProccall2,

                     privateReadInt2,

                     privatesegassign2,            (*5-5-94*)

                     privateSetTime2,
                     privateSimpleAssign2,
                     proccall2,
                     procedure2,
		     program2,
                     pushconst2,
                     pushcounter2,
                     pushname2,
                     pushpattern2,
                     random2,
                     randomize2,
                     readint2,
                     readlnx2,
                     relax2,

                     segassign2,                   (*5-5-94*)
                     segbroadcast2,                (*5-5-94*)
                     segdyninit2,                  (*5-5-94*)
                     segindex2,                    (*5-5-94*)
                     seginit2,                     (*5-5-94*)

                     setTime2,
                     shared2,
		     shl2,
                     shlInt2, 
                     shr2, 
                     shrInt2, 
                     simpleassign2,
		     simplevalue2, 
                     singleControl2, 
                     singlepattern2,

                     size2,                        (*5-5-94*)

                     subtract2,
		     value2, 
                     variable2, 
                     varparam2,
		     writeint2, 
                     writestr2,
		     xor2, 
                     xorInt2 );

  DebugState = (TraceInto, LetGo, StepOver);

  Errors = ( LoadError, AssemError, UntilError, ImpError, MemError,
             NumError, ProgramError, SaveError,ControlRecordError,
             RecordError, FillError);
  ErrorsTable = array[Errors] of string[80];

Const
  ErrorMessages : ErrorsTable = (
    'Error. The syntax is : LOAD Address, FileName.',
    'Error. The syntax is : ASS Address.',
    'Error. The syntax is : UNTIL Address.',
    'Error. The syntax is : IMP Number.',
    'Error. The syntax is : MEM Address, Address.',
    'Error. The syntax is : <NUMBER> [, <NUMBER> ]',
    'Error. The syntax is : PROGRAM Address, HowManyInstructions.',
    'Error. The syntax is : SAVE Address, Address, FileName.',
    'Error. The syntax is : Number, Number, ... , Number.',
    'Error. The syntax is : REC [b][c][p][s].',
    'Error. The syntax is : FILL Address.'
  );

Const
  LastInstruction : OperationPart = xor2;

Type
  Instructions = Set of OperationPart;

Const
  JumpIns : Instructions =
      [ EndProc2, GoIfZero2, Goto2, Program2, Procedure2, Do2, NotDo2 ];

Type
  string20 = string[20];
  NamesTable = array [OperationPart] of string20;
  Displacements = array [OperationPart] of byte;
  Costs = array [OperationPart] of word;

Const
  Disp : Displacements =
         (1  (*active2*),
          1  (*add2*),
          2  (*and2*),
          1  (*andInt2*),
          2  (*assign2*),
          2  (*broadcast*),                        (*5-5-94*)
	  1  (*complement2*),
	  2  (*constant2*),
	  1  (*copyNoTops2*),
	  1  (*copyTops2*),
          1  (*debug2*),
          1  (*defaddr2 *),
          1  (*defarg2 *),
          1  (*divide2*),
          2  (*do2*),
          1  (*endblock2*),
          1  (*endNestedPattern2*),
          1  (*endPattern2*),
          2  (*endproc2*),
          1  (*endprog2*),
          2  (*equal2*),
          1  (*equalInt2*),
          1  (*endRelax2*),
          1  (*endSinglePattern2*),
          2  (*field2*),
	  5  (*fill2*),
	  2  (*fillzero2*),
	  2  (*free2*),
          1  (*getTime2*),
	  2  (*globalcall2*),

	  2  (*globalcalltwo2*),
	  2  (*globalcallthree2*),

	  2  (*globalvalue2*),

	  2  (*globalvaluetwo2*),
	  2  (*globalvaluethree2*),

	  2  (*globalvar2*),

	  2  (*globalvartwo2*),
	  2  (*globalvarthree2*),

	  2  (*goIfNoZero2*),
	  2  (*goIfZero2*),
	  2  (*goto2*),
	  2  (*greater2*),
          1  (*greaterInt2*),
	  2  (*in2*),
	  5  (*index2*),
          2  (*initFor2*),
          2  (*initNestedFor2*),
	  2  (*less2*),
          1  (*lessInt2*),
          2  (*localcall2*),
	  2  (*localvalue2*),
	  2  (*localvar2*),
          3  (*loop*),
	  5  (*marked*),
	  1  (*max2*),
	  1  (*min2*),
	  1  (*Minus2*),
	  1  (*modulo2*),
	  1  (*multiply2*),
          3  (*nestedLoop2*),
	  3  (*nestedpattern2*),
	  2  (*newblock2*),
	  2  (*newline2*),
	  2  (*not2*),
          1  (*notInt2*),
          2  (*notdo2*),
          2  (*notequal2*),
          1  (*notequalInt2*),
          2  (*notgreater2*),
          1  (*notgreaterInt2*),
          2  (*notless2*),
          1  (*notlessInt2*),
          2  (*or2*),
          1  (*orInt2*),
          3  (*pattern2*),
          1  (*popPattern2*),
          2  (*privateAssign2*),
          1  (*privateGetTime2*),

          2  (*privateGlobalCall12*),  (* compilador no las emite *)
          2  (*privateGlobalCall22*),  (**)
          2  (*privateGlobalCall32*),  (**)
          2  (*privateLocalCall2*),    (**)
          2  (*privateNewblock2*),     (**)
          3  (*privateProccall2*),     (* compilador no las emite *)


          1  (*privateReadInt2*),

          3  (*privatesegassign2*),                (*5-5-94*)

          1  (*privateSetTime2*),
          1  (*privateSimpleAssign2*),
          3  (*proccall2*),
          5  (*procedure2*),
	  5  (*program2*),
	  2  (*pushconst2*),
          1  (*pushcounter2*),
	  2  (*pushname2*),
          1  (*pushPattern2*),
          2  (*random*),
          1  (*randomize*),
          1  (*readInt2*),
          1  (*readlnx2*),
          1  (*relax2*),

          3  (*segassign*),                        (*5-5-94*)
          2  (*segbroadcast*),                     (*5-5-94*)
          3  (*segdyninit*),                       (*5-5-94*)
          3  (* segindex2*),                       (*5-5-94*)
          5  (* seginit2 *),                       (*5-5-94*)

          1  (*setTime2*),
          2  (*shared2*),
          2  (*shl2*),
          1  (*shlInt2*),
          2  (*shr*),
          1  (*shrInt2*),
          1  (*simpleassign2*),
	  1  (*simplevalue2*),
	  2  (*singlecontrol2*),
	  2  (*singlepattern2*),

          1  (*size2*),                            (*5-5-94*)

	  1  (*subtract2*),
          2  (*value2*),
          3  (*variable2*),
          3  (*varparam2*),
          1  (*writeint2*),
          3  (*writeStr2*),
          2  (*xor2*),
          1  (*xorInt2*)
          );
Const
  Nc : NamesTable =
                       ('ACTIVE',
                        'ADD',
                        'AND',
                        'ANDINT',
                        'ASSIGN',
                        'BROADCAST',               (*5-5-94*)
            		'COMPLEMENT',
			'CONSTANT',
			'COPYNOTOPS',
                        'COPYTOPS',
                        'DEBUG',
                        'DEFADDR',
                        'DEFARG',
                        'DIVIDE',
                        'DO',
                        'ENDBLOCK',
                        'ENDNESTEDPATTERN',
                        'ENDPATTERN',
                        'ENDPROC',
                        'ENDPROG',
                        'EQUAL',
                        'EQUALINT',
                        'ENDRELAX',
                        'ENDSINGLEPATTERN',
                        'FIELD',
			'FILL',
			'FILLZERO',
			'FREE',
                        'GETTIME',
			'GLOBALCALL',

                        'GLOBALCALLTWO',
                        'GLOBALCALLTHREE',

			'GLOBALVALUE',

			'GLOBALVALUETWO',
			'GLOBALVALUETHREE',

			'GLOBALVAR',

			'GLOBALVARTWO',
			'GLOBALVARTHREE',

                        'GOIFNOZERO',
			'GOIFZERO',
			'GOTO',
			'GREATER',
                        'GREATERINT',
			'IN',
			'INDEX',
                        'INITFOR',
                        'INITNESTEDFOR',
			'LESS',
                        'LESSINT',
                        'LOCALCALL',
			'LOCALVALUE',
			'LOCALVAR',
                        'LOOP',
			'MARKED',
			'MAX',
			'MIN',
			'MINUS',
			'MODULO',
			'MULTIPLY',
                        'NESTEDLOOP',
			'NESTEDPATTERN',
			'NEWBLOCK',
			'NEWLINE',
			'NOT',
                        'NOTINT',
			'NOTDO',
			'NOTEQUAL',
                        'NOTEQUALINT',
			'NOTGREATER',
                        'NOTGREATERINT',
			'NOTLESS',
                        'NOTLESSINT',
                        'OR',
                        'ORINT',
			'PATTERN',
			'POPPATTERN',
                        'PRIVATEASSIGN',
                        'PRIVATEGETTIME',

                        'privateglobalcall12',  (*7-1-94*)
                        'privateGlobalcall22',  (*7-1-94*)
                        'privateGlobalcall32',  (*7-1-94*)
                        'privatelocalcall2',    (*7-1-94*)
                        'privateNewblock2',     (*7-1-94*)
                        'privateProccall2',     (*7-1-94*)

                        'PRIVATEREADINT',

                        'PRIVATESEGASSIGN',        (*5-5-94*)

                        'PRIVATESETTIME',
                        'PRIVATESIMPLEASSIGN',
			'PROCCALL',
			'PROCEDURE',
			'PROGRAM',
			'PUSHCONST',
                        'PUSHCOUNTER',
			'PUSHNAME',
                        'PUSHPATTERN',
                        'RANDOM',
                        'RANDOMIZE',
                        'READINT',
                        'READLN',
                        'RELAX',

                        'SEGASSIGN',               (*5-5-94*)
                        'SEGBROADCAST',            (*5-5-94*)
                        'SEGDYNINIT',              (*5-5-94*)
                        'SEGINDEX',                (*5-5-94*)
                        'SEGINIT',                 (*5-5-94*)

                        'SETTIME',
                        'SHARED',
                        'SHL',
                        'SHLINT',
                        'SHR',
                        'SHRINT',
                        'SIMPLEASSIGN',
			'SIMPLEVALUE',
			'SINGLECONTROL',
			'SINGLEPATTERN',

                        'SIZE',                    (*5-5-94*)

                        'SUBTRACT',
                        'VALUE',
                        'VARIABLE',
                        'VARPARAM',
                        'WRITEINT',
                        'WRITESTR',
                        'XOR',
                        'XORINT'
                        );

  Type

    ProcessorsRange = -1..MaxProcessor;
    Address = MinAddress..MaxAddress;

    ProgramStore = array [Address] of LongInt;

    Processor = record
                  Id : integer;
                  Name : LongInt; { Identifier }
                  b : LongInt; { Base Pointer }
                  s : LongInt; { Stack Pointer }
                  StackBottom : LongInt;
                end;

    SuperRecord = LongInt;

    ActiveProcessors = array[ProcessorsRange] of ProcessorsRange;

  Var
    FreeMemory,
    LocalStackSize,
    MemoryControlSize,
    MinMemoryControlAddress,
    MaxMemoryControlAddress : integer;

    (* FELIX 14.07.92 *)
    MaxNumberOfProcessors, CountSeqTime, CountParTime : LongInt;
    ComplexActive : Boolean;
    (* FELIX 14.07.92 *)

implementation
end.
